//
//  ImageResources.m
//  PlaneGame
//
//  Created by Rannie on 13-10-4.
//  Copyright (c) 2013年 Rannie. All rights reserved.
//

#import "ImageResources.h"

@implementation ImageResources

//singleton
+ (ImageResources *)sharedImages
{
    static dispatch_once_t onceToken;
    static ImageResources *shared;
    dispatch_once(&onceToken, ^{
        shared = [[ImageResources alloc] initSharedImages];
    });
    
    return shared;
}

- (id)init
{
    NSAssert(NO, @"使用单例方法");
    return nil;
}

- (id)initSharedImages
{
    self = [super init];
    if (self)
    {
        NSString *bundlePath = [[[NSBundle mainBundle] bundlePath] stringByAppendingPathComponent:@"images.bundle"];
        NSBundle *imageBundle = [NSBundle bundleWithPath:bundlePath];
        
        //加载背景图片
        self.backgroundImage = [self imageWithImageName:@"background_2" andBundle:imageBundle];
        //加载英雄飞机图片
        self.heroFlyImages = [self imagesWithFormat:@"hero_fly_%d" bundle:imageBundle andCount:2];
        self.heroBlowupImages = [self imagesWithFormat:@"hero_blowup_%d" bundle:imageBundle andCount:4];
        
        //子弹图片
        self.normalBulletImage = [self imageWithImageName:@"bullet1" andBundle:imageBundle];
        //增强子弹图片
        self.enhancedBulletImage = [self imageWithImageName:@"bullet2" andBundle:imageBundle];
        
        //敌机照片
        self.smallFlyImage = [self imageWithImageName:@"enemy1_fly_1" andBundle:imageBundle];
        self.smallBlowupImages = [self imagesWithFormat:@"enemy1_blowup_%d" bundle:imageBundle andCount:4];
        
        self.middleFlyImage = [self imageWithImageName:@"enemy3_fly_1" andBundle:imageBundle];
        self.middleHitImage = [self imageWithImageName:@"enemy3_hit_1" andBundle:imageBundle];
        self.middleBlowupImages = [self imagesWithFormat:@"enemy3_blowup_%d" bundle:imageBundle andCount:4];
        
        self.bigFlyImages = [self imagesWithFormat:@"enemy2_fly_%d" bundle:imageBundle andCount:2];
        self.bigHitImage = [self imageWithImageName:@"enemy2_hit_1" andBundle:imageBundle];
        self.bigBlowupImages = [self imagesWithFormat:@"enemy2_blowup_%d" bundle:imageBundle andCount:7];
    }
    
    return self;
}

- (UIImage *)imageWithImageName:(NSString *)imageName andBundle:(NSBundle *)bundle
{
    NSString *imagePath = [bundle pathForResource:imageName ofType:@"png"];
    UIImage *image = [UIImage imageWithContentsOfFile:imagePath];
    
    return image;
}

- (NSArray *)imagesWithFormat:(NSString *)format bundle:(NSBundle *)bundle andCount:(NSInteger)count
{
    NSMutableArray *images = [NSMutableArray arrayWithCapacity:count];
    for (NSInteger i = 1; i <= count; i++)
    {
        NSString *imageName = [NSString stringWithFormat:format, i];
        NSString *imagePath = [bundle pathForResource:imageName ofType:@"png"];
        UIImage *image = [UIImage imageWithContentsOfFile:imagePath];
        
        [images addObject:image];
    }
    
    return images;
}

@end
