//
//  PlaneGameViewController.m
//  PlaneGame
//
//  Created by Rannie on 13-10-4.
//  Copyright (c) 2013年 Rannie. All rights reserved.
//

#import "PlaneGameViewController.h"

#import <QuartzCore/QuartzCore.h>

#import "ImageResources.h"
#import "SoundsTool.h"
#import "GameModel.h"
#import "Bullet.h"
#import "Hero.h"

#import "BackgroundView.h"
#import "HeroView.h"
#import "BulletView.h"
#import "EnemyView.h"

#define RBulletMoveStep 8.0f

#define RMaxLevel 10

static long long _steps;

@interface PlaneGameViewController () <UIAlertViewDelegate>

//游戏模型
@property (strong, nonatomic) GameModel *model;
//游戏视图
@property (strong, nonatomic) UIView *gameView;
//控制视图
@property (strong, nonatomic) UIView *controlView;
//背景图像
@property (weak, nonatomic) BackgroundView *bgView;
//游戏时钟
@property (strong, nonatomic) CADisplayLink *gameTimer;
//英雄视图
@property (strong, nonatomic) HeroView *heroView;
//子弹视图集合
@property (strong, nonatomic) NSMutableSet *bulletViewSet;
//敌机集合
@property (strong, nonatomic) NSMutableSet *enemyViewSet;
//游戏计时板
@property (weak, nonatomic) UILabel *clockLabel;
//游戏得分
@property (assign, nonatomic) NSInteger totalScore;
//得分板
@property (weak, nonatomic) UILabel *scoreLabel;

@end

@implementation PlaneGameViewController

- (void)viewDidLoad
{
    [super viewDidLoad];
    
    _steps = 0;
    
    self.bulletViewSet = [NSMutableSet set];
    self.enemyViewSet = [NSMutableSet set];
    
    //游戏视图
    UIView *gameView = [[UIView alloc] initWithFrame:self.view.bounds];
    [self.view addSubview:gameView];
    self.gameView = gameView;
    
    //设置模型
    [self setGameModel];
    
    //控制视图
    UIView *controlView = [[UIView alloc] initWithFrame:self.view.bounds];
    controlView.backgroundColor = [UIColor clearColor];
    [self.view addSubview:controlView];
    self.controlView = controlView;
    
    //时钟标签
    UILabel *label = [[UILabel alloc] initWithFrame:CGRectMake(10, self.view.frame.size.height - 30, 100, 20)];
    label.backgroundColor = [UIColor clearColor];
    label.text = @"00:00";
    [self.controlView addSubview:label];
    self.clockLabel = label;
    
    //暂停继续按钮
    UIButton *controlButton = [UIButton buttonWithType:UIButtonTypeCustom];
    controlButton.frame = CGRectMake(10, 10, 40, 30);
    [controlButton setImage:[UIImage imageNamed:@"images.bundle/BurstAircraftPause"] forState:UIControlStateNormal];
    [controlButton setImage:[UIImage imageNamed:@"images.bundle/BurstAircraftPauseHL"] forState:UIControlStateHighlighted];
    [controlButton addTarget:self action:@selector(controlGame:) forControlEvents:UIControlEventTouchUpInside];
    [self.controlView addSubview:controlButton];
    
    //得分板
    UILabel *sLabel = [[UILabel alloc] initWithFrame:CGRectMake(50, 10, 270, 30)];
    sLabel.font = [UIFont fontWithName:@"Marker Felt" size:24.0f];
    sLabel.textColor = [UIColor darkGrayColor];
    sLabel.backgroundColor = [UIColor clearColor];
    [self.controlView addSubview:sLabel];
    self.scoreLabel = sLabel;
    
    [self startGameTimer];
}

#pragma mark -
#pragma mark Control Method

- (void)setGameModel
{
    CGSize heroSize = [[ImageResources sharedImages].heroFlyImages[0] size];
    if (self.model)
    {
        self.model = nil;
    }
    self.model = [GameModel gameModelWithGameArea:self.gameView.bounds heroSize:heroSize];
    
    self.model.gameLevel = 1;
    
    self.model.hero.bulletNormalSize = [ImageResources sharedImages].normalBulletImage.size;
    self.model.hero.bulletEnhancedSize = [ImageResources sharedImages].enhancedBulletImage.size;
    
    //背景视图
    if (self.bgView)
    {
        [self.bgView removeFromSuperview];
        self.bgView = nil;
    }
    BackgroundView *bgView = [[BackgroundView alloc] initWithFrame:self.view.bounds andImage:[ImageResources sharedImages].backgroundImage];
    [self.gameView addSubview:bgView];
    self.bgView = bgView;
    
    //英雄飞机视图
    if (self.heroView)
    {
        [self.heroView removeFromSuperview];
        self.heroView = nil;
    }
    HeroView *heroView = [[HeroView alloc] initWithHeroImages:[ImageResources sharedImages].heroFlyImages];
    heroView.center = self.model.hero.position;
    [self.gameView addSubview:heroView];
    self.heroView = heroView;
}

- (void)controlGame:(UIButton *)sender
{
    NSInteger i = sender.tag = !sender.tag;
    MyLog(@"暂停tag:%d", i);
    
    if (i)
    {
        //暂停游戏
        [self pauseGameTimer];
        [sender setImage:[UIImage imageNamed:@"images.bundle/BurstAircraftStart"] forState:UIControlStateNormal];
        [sender setImage:[UIImage imageNamed:@"images.bundle/BurstAircraftStartHL"] forState:UIControlStateHighlighted];
    }
    else
    {
        //继续游戏
        [self startGameTimer];
        [sender setImage:[UIImage imageNamed:@"images.bundle/BurstAircraftPause"] forState:UIControlStateNormal];
        [sender setImage:[UIImage imageNamed:@"images.bundle/BurstAircraftPauseHL"] forState:UIControlStateHighlighted];
    }
}

- (void)startGameTimer
{
    self.gameTimer = [CADisplayLink displayLinkWithTarget:self selector:@selector(steps)];
    [self.gameTimer addToRunLoop:[NSRunLoop mainRunLoop] forMode:NSDefaultRunLoopMode];
    
    [[SoundsTool shareSoundsTool] playMusic];
}

- (void)pauseGameTimer
{
    [self.gameTimer invalidate];
    
    [[SoundsTool shareSoundsTool] pauseMusic];
}

- (void)endGame
{
    UIAlertView *alert = [[UIAlertView alloc] initWithTitle:@"游戏结束"
                                                    message:[NSString stringWithFormat:@"您的得分为:%d000\n点击按钮重新开始游戏", self.totalScore]
                                                   delegate:self
                                          cancelButtonTitle:@"重新开始"
                                          otherButtonTitles:nil];
    [alert show];
    
    [[SoundsTool shareSoundsTool] stopMusic];
}

- (void)resetGame
{
    self.totalScore = 0;
    self.clockLabel.text = @"00:00";
    self.scoreLabel.text = nil;
    
    _steps = 0;
    
    [self setGameModel];
    
    for (EnemyView *enemyView in self.enemyViewSet)
    {
        [enemyView removeFromSuperview];
    }
    [self.enemyViewSet removeAllObjects];
    
    for (BulletView *bulletView in self.bulletViewSet)
    {
        [bulletView removeFromSuperview];
    }
    [self.bulletViewSet removeAllObjects];
}

- (void)alertView:(UIAlertView *)alertView clickedButtonAtIndex:(NSInteger)buttonIndex
{
    [self resetGame];

    if (buttonIndex == 0)
    {
        [self startGameTimer];
    }
}

- (void)loadResource
{
    [ImageResources sharedImages];
    [SoundsTool shareSoundsTool];
}

- (void)updateScoreLabel
{
    self.scoreLabel.text = [NSString stringWithFormat:@"%d000", self.totalScore];
    
    if (self.totalScore/self.model.gameLevel > 300)
    {
        if (self.model.gameLevel == RMaxLevel)
        {
            self.model.gameLevel = RMaxLevel;
        }
        else
        {
            self.model.gameLevel++;
        }
    }
}

#pragma mark -
#pragma mark StepMethod

- (void)steps
{
    _steps++;
    
    if (_steps % 60 == 0)
    {
        [self updateGameClock];
    }
    
    //背景处理
    [self.model moveBackground];
    [self.bgView setBackgroundFrame1:self.model.bgFrame1 andFrame2:self.model.bgFrame2];
    
    //英雄视图位置刷新
    self.heroView.center = self.model.hero.position;
    
    //英雄飞机射击
    if (_steps % 10 == 0)
    {
        [self.model.hero fire];
    }
    
    //射击子弹处理
    [self checkBullets];
    
    //初始化敌机
    [self initialEnemies];
    
    //敌机移动
    [self updateEnemies];
    
    //碰撞检测
    [self collisionDetector];
}

#pragma mark -
#pragma mark Collision

- (void)collisionDetector
{
    /******子弹碰撞敌方飞机******/
    NSMutableSet *removeBulletSet = [NSMutableSet set];
    for (BulletView *bulletView in self.bulletViewSet)
    {
        Bullet *bullet = bulletView.bullet;
        
        for (EnemyView *enemyView in self.enemyViewSet)
        {
            Enemy *enemy = enemyView.enemy;
            
            if (CGRectIntersectsRect(bulletView.frame, enemyView.frame) && !enemy.toBlowup)
            {
                enemy.hp -= bullet.damage;
                
                if (enemy.hp <= 0)
                {
                    enemy.toBlowup = YES;
                }
                else
                {
                    if (enemy.type == REnemyTypeBig)
                    {
                        [enemyView stopAnimating];
                    }
                    enemyView.image = enemyView.hitImage;
                }
                [removeBulletSet addObject:bulletView];
            }
        }
    }
    
    for (BulletView *bulletView in removeBulletSet)
    {
        [bulletView removeFromSuperview];
        [self.bulletViewSet removeObject:bulletView];
    }
    
    /******爆炸处理******/
    if (_steps % 4 == 0)
    {
        NSMutableSet *toRemovedSet = [NSMutableSet set];
        
        for (EnemyView *enemyView in self.enemyViewSet)
        {
            Enemy *enemy = enemyView.enemy;
            
            if (enemy.toBlowup)
            {
                enemy.speed = 0;
                enemyView.image = enemyView.blowupImages[enemy.blowupFrames++];
            }
            
            if (enemy.blowupFrames == enemyView.blowupImages.count)
            {
                [toRemovedSet addObject:enemyView];
            }
        }
        
        for (EnemyView *enemyView in toRemovedSet)
        {
            self.totalScore += enemyView.enemy.prize;
            [self updateScoreLabel];
            
            [self.enemyViewSet removeObject:enemyView];
            [enemyView removeFromSuperview];
        }
        [toRemovedSet removeAllObjects];
    }
    
    /******敌机碰撞英雄飞机******/
    for (EnemyView *enemyView in self.enemyViewSet)
    {
        if (CGRectIntersectsRect(enemyView.frame, self.model.hero.collisionRect))
        {
            [self.heroView stopAnimating];
            
            NSArray *images = [ImageResources sharedImages].heroBlowupImages;
            
            self.heroView.image = images[3];
            self.heroView.animationImages = images;
            self.heroView.animationDuration = 1.0f;
            self.heroView.animationRepeatCount = 1;
            
            [self.heroView startAnimating];
            
            [self pauseGameTimer];
            MyLog(@"英雄死亡");
            
            [[SoundsTool shareSoundsTool] playSoundWithType:RSoundTypeHeroOver];
            
            [self endGame];
        }
    }
}

#pragma mark -
#pragma mark Enemies Method

- (void)initialEnemies
{
    NSInteger level = self.model.gameLevel;
    NSInteger flag = RMaxLevel/level;
    
    NSInteger timeS = arc4random_uniform(10*flag)+10/level+flag*1;
    NSInteger timeM = arc4random_uniform(50*flag)+200/level+flag*3;
    NSInteger timeB = arc4random_uniform(100*flag)+1000/level+flag*10;
    
    Enemy *enemy = nil;
    if (_steps % timeS == 0)
    {
        enemy = [self.model createEnemyWithType:REnemyTypeSmall andSize:[ImageResources sharedImages].smallFlyImage.size];
        [self initEnemyViewWithEnemy:enemy];
    }
    else if (_steps % timeM == 0)
    {
        enemy = [self.model createEnemyWithType:REnemyTypeMiddle andSize:[ImageResources sharedImages].middleFlyImage.size];
        [self initEnemyViewWithEnemy:enemy];
    }
    else if (_steps % timeB == 0)
    {
        enemy = [self.model createEnemyWithType:REnemyTypeBig andSize:[[ImageResources sharedImages].bigFlyImages[0] size]];
        [self initEnemyViewWithEnemy:enemy];
        
        [[SoundsTool shareSoundsTool] playSoundWithType:RSoundTypeBigFly];
    }
}

- (void)initEnemyViewWithEnemy:(Enemy *)enemy
{
    EnemyView *enemyView = [[EnemyView alloc] initWithEnemy:enemy];
    
    [self.gameView addSubview:enemyView];
    [self.enemyViewSet addObject:enemyView];
}

- (void)updateEnemies
{
    NSMutableSet *removeEnemySet = [NSMutableSet set];
    
//    MyLog(@"敌机数量:%d", self.enemyViewSet.count);
    
    for (EnemyView *enemyView in self.enemyViewSet)
    {
        Enemy *enemy = enemyView.enemy;
        enemy.position = CGPointMake(enemy.position.x, enemy.position.y + enemy.speed);
        
        enemyView.center = enemy.position;
        
        if (CGRectGetMinY(enemyView.frame) > self.view.bounds.size.height)
        {
            [removeEnemySet addObject:enemyView];
        }
    }
    
    for (EnemyView *enemyView in removeEnemySet)
    {
        [self.enemyViewSet removeObject:enemyView];
        [enemyView removeFromSuperview];
    }
    [removeEnemySet removeAllObjects];
}

#pragma mark -
#pragma mark Bullet Method

- (void)checkBullets
{
    NSMutableSet *removeBulletSet = [NSMutableSet set];
    
//    MyLog(@"子弹集合数量:%d", self.bulletViewSet.count);
    
    for (BulletView *bulletView in self.bulletViewSet)
    {
        CGPoint center = CGPointMake(bulletView.center.x, bulletView.center.y - RBulletMoveStep);
        bulletView.center = center;
        
        if (CGRectGetMaxY(bulletView.frame) <= 0)
        {
            [removeBulletSet addObject:bulletView];
        }
    }
    
    for (BulletView *bulletView in removeBulletSet)
    {
        [self.bulletViewSet removeObject:bulletView];
        [bulletView removeFromSuperview];
    }
    [removeBulletSet removeAllObjects];
    
    if (self.model.hero.currentBullet)
    {
        Bullet *bullet = self.model.hero.currentBullet;
        UIImage *bulletImage = nil;
        if (bullet.isEnhanced)
        {
            bulletImage = [ImageResources sharedImages].enhancedBulletImage;
        }
        else
        {
            bulletImage = [ImageResources sharedImages].normalBulletImage;
        }
        
        BulletView *bulletView = [[BulletView alloc] initWithImage:bulletImage andBullet:bullet];
        bulletView.center = bullet.position;
        bulletView.alpha = 0.6;
        [self.gameView addSubview:bulletView];
        [self.bulletViewSet addObject:bulletView];
        
        [[SoundsTool shareSoundsTool] playSoundWithType:RSoundTypeBullet];
        
        self.model.hero.currentBullet = nil;
    }
}

- (void)touchesMoved:(NSSet *)touches withEvent:(UIEvent *)event
{
    UITouch *touch = [touches anyObject];
    
    CGPoint loc = [touch locationInView:self.view];
    CGPoint preLoc = [touch previousLocationInView:self.view];
    
    CGPoint offset = CGPointMake(loc.x - preLoc.x, loc.y - preLoc.y);
    CGPoint pos = self.model.hero.position;
    CGPoint newPos = CGPointMake(pos.x + offset.x, pos.y + offset.y);
    
    CGSize heroSize = self.model.hero.size;
    CGSize areaSize = self.model.gameArea.size;
    
    if (newPos.x <= heroSize.width/2)
    {
        newPos.x = heroSize.width/2;
    }
    else if (newPos.x >= areaSize.width - heroSize.width/2)
    {
        newPos.x = areaSize.width - heroSize.width/2;
    }
    
    if (newPos.y <= heroSize.height/2)
    {
        newPos.y = heroSize.height/2;
    }
    else if (newPos.y >= areaSize.height - heroSize.height/2)
    {
        newPos.y = areaSize.height - heroSize.height/2;
    }
    
    self.model.hero.position = newPos;
}

- (void)updateGameClock
{
    NSString *timeText = [NSString stringWithFormat:@"%02d:%02d", (NSInteger)(_steps/60)/60,(NSInteger)(_steps/60)%60];
    self.clockLabel.text = timeText;
}

@end
