//
//  SoundsTool.m
//  PlaneGame
//
//  Created by Rannie on 13-10-6.
//  Copyright (c) 2013年 Rannie. All rights reserved.
//

#import "SoundsTool.h"

#import <AVFoundation/AVFoundation.h>
#import <AudioToolbox/AudioToolbox.h>

@interface SoundsTool ()

@property (strong, nonatomic) AVAudioPlayer *musicPlayer;
@property (strong, nonatomic) NSDictionary *soundDict;

@end

@implementation SoundsTool

+ (id)shareSoundsTool
{
    static dispatch_once_t onceToken;
    static SoundsTool *sharedTool;
    dispatch_once(&onceToken, ^{
        sharedTool = [[SoundsTool alloc] initSharedTool];
    });
    
    return sharedTool;
}

- (id)init
{
    NSAssert(NO, @"使用单例方法");
    return nil;
}

#pragma mark -
#pragma mark Private Method

- (id)initSharedTool
{
    self = [super init];
    
    if (self)
    {
        NSString *bundlePath = [[[NSBundle mainBundle] bundlePath] stringByAppendingPathComponent:@"music.bundle"];
        NSBundle *bundle = [NSBundle bundleWithPath:bundlePath];
        NSString *filePath = [bundle pathForResource:@"game_music" ofType:@"mp3"];
        NSURL *musicURL = [NSURL fileURLWithPath:filePath];
        
        [self loadSoundDictWithBundle:bundle];
        
        NSError *error;
        self.musicPlayer = [[AVAudioPlayer alloc] initWithContentsOfURL:musicURL error:&error];
        if (error)
        {
            MyLog(@"音乐播放器初始化失败,错误:%@", error);
        }
        
        self.musicPlayer.numberOfLoops = -1;
        [self.musicPlayer prepareToPlay];
    }
    
    return self;
}

- (void)loadSoundDictWithBundle:(NSBundle *)bundle
{
    NSMutableDictionary *dictM = [NSMutableDictionary dictionary];
    
    NSArray *soundsArray = @[@"bullet",
                             @"enemy1_down",
                             @"enemy2_down",
                             @"enemy2_out",
                             @"enemy3_down",
                             @"game_over"];
    for (NSString *soundName in soundsArray)
    {
        SystemSoundID soundID = [self soundIdWithName:soundName andBundle:bundle];
        
        [dictM setObject:@(soundID) forKey:soundName];
    }
    
    self.soundDict = dictM;
}

- (SystemSoundID)soundIdWithName:(NSString *)name andBundle:(NSBundle *)bundle
{
    SystemSoundID soundID;
    
    NSString *soundPath = [bundle pathForResource:name ofType:@"mp3"];
    NSURL *soundURL = [NSURL fileURLWithPath:soundPath];
    
    AudioServicesCreateSystemSoundID((__bridge CFURLRef)soundURL, &soundID);
    
    return soundID;
}

#pragma mark -
#pragma mark Action Method

- (void)playMusic
{
    [self.musicPlayer play];
}

- (void)pauseMusic
{
    [self.musicPlayer pause];
}

- (void)stopMusic
{
    [self.musicPlayer stop];
}

- (void)playSoundWithType:(RSoundType)soundType
{
    NSString *soundName = nil;
    switch (soundType)
    {
        case RSoundTypeBullet:
            soundName = @"bullet";
            break;
        case RSoundTypeSmallBlowup:
            soundName = @"enemy1_down";
            break;
        case RSoundTypeMiddleBlowup:
            soundName = @"enemy3_down";
            break;
        case RSoundTypeBigFly:
            soundName = @"enemy2_out";
            break;
        case RSoundTypeBigBlowup:
            soundName = @"enemy2_down";
            break;
        case RSoundTypeHeroOver:
            soundName = @"game_over";
            break;
    }
    
    [self playSoundWithSoundName:soundName];
}

- (void)playSoundWithSoundName:(NSString *)soundName
{
    SystemSoundID soundId = [self.soundDict[soundName] unsignedLongValue];
    
    AudioServicesPlaySystemSound(soundId);
}

@end
